/************************************************************
**	File:	prf636dll.h
**	Author:	
**			AES, HJ
**		      
**	Description:
**		prototype and structures file for prf636.dll
**      
**	Revision history:
**		Original:	06/22/12
**					07/09/12 - added function descriptions etc
**					07/16/12 - revised after feedback
**					07/29/12 - merged definitions
**      
**	THIS IS THE UNPUBLISHED PROPRIETARY SOURCE CODE OF CONNECT TECH INC.
**	The copyright notice above does not evidence any actual or intended
**	publication of such source code.
**      
**	This module contains Proprietary Information of Connect Tech, Inc
**	and should be treated as Confidential.
************************************************************/

//
// the following definition is from dll to application
//
#define J1708_STATUS_OK								0x0000
#define J1708_STATUS_INVALIDPARAM					0x0001
#define J1708_STATUS_IOFAILURE						0x0002
#define J1708_STATUS_NOT_INIT						0x0003
#define J1708_STATUS_INSUFFICIENT_BUFFER			0x0004

#define J1939_STATUS_OK								0x0000
#define J1939_STATUS_INVALIDPARAM					0x0001
#define J1939_STATUS_IOFAILURE						0x0002
#define J1939_STATUS_NOT_INIT						0x0003
#define J1939_STATUS_INSUFFICIENT_BUFFER			0x0004

typedef DWORD  J1708_STATUS;
typedef DWORD  J1939_STATUS;



//
// the following definition is from driver to application via dll
//
//
// receive buffer overrun in the driver
//
#define BUF_OVERRUN					0xb0000001
//
// J1939 stack rejected packet to transmit
//
#define STATUS_J1939_TX_FAILED		0xb0000002

// GetWriteStatusJ1708 values. For more detail refer to hardware manual section 7.1.2.1
//
// this indicates that there is a TX abort error on J1708
//
#define STATUS_J1708_TX_ABORTED		0xb0000040
//
// this indicates that there is a TX problem error on J1708
//
#define STATUS_J1708_TX_PROB		0xb0000080

//
// other difinitions
//
#define	MAX_J1939_SIZE			1785
#define MAX_J1939_CONTROL_SIZE	126
//#define MAX_J1708_SIZE			There is no max J1708 size

// J1939 packet type definitions
#define	J1939_DATA_PACKET			0
#define	J1939_CONTROL_PACKET		1
#define	MCP2515_CAN_DATA_PACKET		2	
#define	MCP2515_CAN_CONTROL_PACKET	3	

// J1939 control packet to set J1939 device name
#define J1939_SET_NAME_CMD 0
// J1939 control packet to get device name
#define J1939_GET_NAME_CMD 1
// J1939 control packet to set address
#define J1939_GET_ADDRESS_CMD 2
// J1939 control packet to get address
#define J1939_SET_ADDRESS_CMD 3

// pack the data using byte alignment to ensure driver stays aligned
#pragma pack(1)

// see section 7.2 of hardware manual for more detailed parameter
// descriptions.  
typedef struct _USER_TA_DLL {
	UCHAR Value;
	UCHAR In_Effect;
	UCHAR Ignore;
}USER_TA_DLL;

typedef struct _J1708_CONFIG {
	ULONG mask_good_tx;
	ULONG Tx_Priority;			// 0-7
	ULONG Tx_Problem_Limit;		//2-254
	ULONG Rx_EOM_Level;			//0-15
	ULONG Rx_Afull_Level;		//1-31	
}J1708_CONFIG;

typedef struct {
   ULONG pgn;					/* parameter group number */
   UCHAR *buf_ptr;				/* set to NULL, used internally in firmware */
   USHORT buf_len;				/* size of data */
   UCHAR dst;					/* destination of message */
   UCHAR src;					/* source of message */
   UCHAR pri;					/* priority of message */
   UCHAR port;					/* CAN port of message */
   UCHAR buf[MAX_J1939_SIZE];	/* actual data */
} cti_j1939_t;

typedef struct {
	UCHAR		command;	// command index number
	UCHAR		data[MAX_J1939_CONTROL_SIZE];	// data associated with command
} cti_j1939_command_dll_t;
#pragma pack()

enum J1708_ports_t { J1708_PORT1 = 1, J1708_PORT2 = 2};
enum J1939_ports_t { J1939_PORT1 = 1, J1939_PORT2 = 2, J1939_PORT3 = 3, J1939_PORT4 = 4};

/***************************************************************************
* OpenJ1708 
* 
* Opens a J1708 port returning a handle for use with other J1708 functions
* Non-blocking
*
* Arguments:
*	index	index to device number (1 = first port)
*
* Returns:
*   INVALID_HANDLE_VALUE on failure otherwise device handle on success
*   
***************************************************************************/
HANDLE _stdcall OpenJ1708(int index );

/***************************************************************************
* CloseJ1708 
* 
* Closes a J1708 port previously opened using OpenJ1708 and frees resources
* Non-blocking
*
* Arguments:
*	handle	handle to device returned on OpenJ1708
*
* Returns:
*   nonzero on success
*   
***************************************************************************/
BOOL _stdcall CloseJ1708(HANDLE  handle);

/***************************************************************************
* SetupJ1708 
* 
* Configures a J1708 port parameters
* Non-blocking
*
* Arguments:
*	devHandle		handle to device returned on OpenJ1708
*	*j1708Config	pointer to configuration structure
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall SetupJ1708(HANDLE devHandle, J1708_CONFIG *j1708Config);

/***************************************************************************
* ReadJ1708 
* 
* Reads from a J1708 port
* Blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer to read to
*	bufferSize	size of buffer in bytes
*	*numRead	on return contains the actual number of bytes read
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall ReadJ1708(HANDLE devHandle, char *buffer, int bufferSize, int *numRead);

/***************************************************************************
* WriteJ1708 
* 
* Writes to a J1708 port
* Blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer to write
*	bufferSize	number of bytes in buffer to write
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall WriteJ1708(HANDLE devHandle, char *buffer, int bufferSize);

/***************************************************************************
* GetReadDataAvailJ1708 
* 
* Get number of bytes available to read on a J1708 port
* Non-blocking
*
* Arguments:
*	devHandle		handle to device returned on OpenJ1708
*	*dataAvail		on return contains number of bytes ready to read
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall GetReadDataAvailJ1708(HANDLE devHandle, int *dataAvail);

/***************************************************************************
* GetReadStatusJ1708 
* 
* Get read status of a J1708 port
* Non-blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer to receive status information
*				status values include BUF_OVERRUN
*	bufferSize	size in bytes of buffer (must be at least 4)
*
* Note in future this function may be extended to return more information
* if necessary, so this is why the buffer is not a pointer to just a ULONG
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall GetReadStatusJ1708(HANDLE devHandle, char *buffer, int bufferSize);


/***************************************************************************
* GetWriteStatusJ1708 
* 
* Get write status of a J1708 port
* Non-blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer to receive status information
*				status values include STATUS_J1708_TX_ABORTED and
*				STATUS_J1708_TX_PROB
*	bufferSize	size in bytes of buffer (must be at least 4)
*
* Note in future this function may be extended to return more information
* if necessary, so this is why the buffer is not a pointer to just a ULONG
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall GetWriteStatusJ1708(HANDLE devHandle, char *buffer, int bufferSize);



/***************************************************************************
* AbortJ1708TX 
* 
* Abort a transmit in progress on a port
* Non-blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall AbortJ1708TX(HANDLE devHandle);

/***************************************************************************
* Write_USER_TA_J1708 
* 
* Set USER_TA values on a J1708 port
* Non-blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*userTA		pointer to USER_TA values to set
*
* Returns:
*   J1708_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall Write_USER_TA_J1708(HANDLE devHandle, USER_TA_DLL *userTA);

/***************************************************************************
* OpenJ1939 
* 
* Opens a J1939 port
* Non-blocking
*
* Arguments:
*	index	index to device number (1 = first port)
*
* Returns:
*   INVALID_HANDLE_VALUE on failure otherwise device handle on success
*   
***************************************************************************/
HANDLE _stdcall OpenJ1939(int index);

/***************************************************************************
* CloseJ1939 
* 
* Closes a J1939 port returning a handle for use with other J1939 functions
* Non-blocking
*
* Arguments:
*	handle	handle to device returned on OpenJ1939
*
* Returns:
*   nonzero on success
*   
***************************************************************************/
BOOL _stdcall CloseJ1939(HANDLE  handle );

/***************************************************************************
* ReadJ1939 
* 
* Reads from a J1939 port
* Blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1939
*	*buffer		pointer to cti_j1939_t buffer to read to
*	bufferSize	size of buffer in bytes
*	*numRead	on return contains the actual number of bytes read
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall ReadJ1939(HANDLE devHandle, cti_j1939_t *buffer, int bufferSize, int *numRead);

/***************************************************************************
* WriteJ1939 
* 
* Writes to a J1939 port
* Blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1939
*	*buffer		pointer to cti_j1939_t buffer to write
*	bufferSize	number of bytes in buffer to write
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall WriteJ1939(HANDLE devHandle, cti_j1939_t *buffer, int bufferSize);

/***************************************************************************
* GetReadDataAvailJ1939 
* 
* Get number of bytes available to read on a J1939 port
* Non-blocking
*
* Arguments:
*	devHandle		handle to device returned on OpenJ1939
*	*dataAvail		on return contains number of bytes ready to read
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall GetReadDataAvailJ1939(HANDLE devHandle, int *dataAvail);

/***************************************************************************
* GetReadStatusJ1939 
* 
* Get read status of a J1939 port
* Non-blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer to receive status information
*				set to BUF_OVERRUN upon buffer overrun
*	bufferSize	size in bytes of buffer (must be at least 4)
*
* Note in future this function may be extended to return more information
* if necessary, so this is why the buffer is not a pointer to just a ULONG
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall GetReadStatusJ1939(HANDLE devHandle, char *buffer, int bufferSize);

/***************************************************************************
* GetWriteStatusJ1939 
* 
* Get write status of a J1939 port
* Non-blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer to receive status information
*				currently no write status on J1939 but here for possible future
*	bufferSize	size in bytes of buffer (must be at least 4)
*
* Note in future this function may be extended to return more information
* if necessary, so this is why the buffer is not a pointer to just a ULONG
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall GetWriteStatusJ1939(HANDLE devHandle, char *buffer, int bufferSize);

/***************************************************************************
* WriteJ1939Name 
* 
* Set name of a J1939 port to NVRAM
* Blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	*buffer		pointer to buffer containing name for port
*	bufferSize	size in bytes of buffer (maximum 8)
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
int _stdcall WriteJ1939Name(HANDLE devHandle, char *buffer, int bufferSize);

/***************************************************************************
* WriteJ1939Control 
* 
* Send a user defined J1939 control packet to NVRAM
* Blocking
*
* Arguments:
*	devHandle	handle to device returned on OpenJ1708
*	buffer		cti_j1939_command_dll_t containing user defined data
*
* Returns:
*   J1939_STATUS_OK on success, otherwise statuses defined in api return statuses
*   
***************************************************************************/
 int _stdcall WriteJ1939Control(HANDLE devHandle, cti_j1939_command_dll_t buffer);

/***************************************************************************
* WriteJ1939GetControl
*
* This routine gets a J1939 Control packet from J1939 Stack
*
* Arguments:
*	devHandle - HANDLE to the device
*	inBuffer  cti_j1939_command_dll_t containing control command to J1939 Stack
*	outBuffer  pointer to cti_j1939_command_dll_t from J1939 stack
*
* Returns:
*   J1939_STATUS_OK if the operation succeeds
*   
*/
 int _stdcall WriteJ1939GetControl(HANDLE devHandle, cti_j1939_command_dll_t inBuffer, cti_j1939_command_dll_t *outBuffer);