/************************************************************
**	File:	samplej1708.cpp
**	Author:	
**			HJ
**		      
**	Description:
**		Sample application for Xtreme J1708 port
**		The sample uses a prf636.dll to communicate to the driver.
**      It needs prf636.lib to compile. 
**
**         
**	THIS IS THE UNPUBLISHED PROPRIETARY SOURCE CODE OF CONNECT TECH INC.
**	The copyright notice above does not evidence any actual or intended
**	publication of such source code.
**      
**	This module contains Proprietary Information of Connect Tech, Inc
**	and should be treated as Confidential.
************************************************************/


#include "stdafx.h"
#include <windows.h>
#include <winioctl.h>
#include <conio.h>
#include <strsafe.h>
#include "prf636dll.h"


//#define USE_FILE 

ULONG	BUF_SIZE = 16;		// default request size
#define MAX_BUF_SIZE 1024	// maximum limit of buffer allocation for read and write

// structure/type used to communicate thread parameters
typedef struct
{
	HANDLE			hDriver;
	int				port;
	ULONG			delay;
}UNIT_CONTEXT, *PUNIT_CONTEXT;

typedef DWORD			uint32_t;
typedef unsigned char	uint8_t;
typedef WORD			uint16_t;


DWORD WINAPI
 ReadThreadJ1708(
    PUNIT_CONTEXT	pContext
    );

DWORD WINAPI
 WriteThreadJ1708(
    PUNIT_CONTEXT	pContext
    );


/***************************************************************************
* main 
*
* This is the main routine that opens and initializes a  unit then launches
* read and/or write thread(s)
*    
*/

int main(int argc, char* argv[])
{
	HANDLE			hWriteThread;
	HANDLE			hReadThread;
	DWORD			dwThreadStatId;
	UNIT_CONTEXT	threadContext;		
	int				unit;
	J1708_CONFIG	j1708Config = {0};	
	BOOL			receiveOnly = FALSE;
	BOOL			transmitOnly = FALSE;

	// in this program parameters are provided on command line syntax is: samplej1708 unit delay packet_size [r/t]
	// where unit is the unit port number 1 = 1st port
	// delay is the delay between packet transmitions in ms
	// packet_size is the size of each packet
	// optional r or t for receive only or transmit only
	if(argc > 3)
	{
		unit=atoi(argv[1]);
		threadContext.delay = atoi(argv[2]);
		BUF_SIZE = atoi(argv[3]);
	}
	else
	{
		printf("samplej1708 unit delay packet_size r/t\nunit= board index(starting 1)\ndelay= in ms between transmit\npacket_size= number of bytes in packet\nr for receive only/t for transmit or neither for both\nUsage: samplej1708 1 0 100\nThe above will run J1708 for port 1 with delay 0 ms 100 byte packet size and receive/transmit on port\n");
		return 1;
	}

	if(BUF_SIZE > MAX_BUF_SIZE)
		BUF_SIZE = MAX_BUF_SIZE;

	if(argc > 4)
		if( *argv[4] == 't' )
			transmitOnly = TRUE;
		else if( *argv[4] == 'r' )
			receiveOnly = TRUE;
	
	threadContext.port = unit;	

	threadContext.hDriver = OpenJ1708( unit );
	if (threadContext.hDriver == INVALID_HANDLE_VALUE) 
	{		
		printf("unit %d J1708 could not be opened, error %x\n", unit, GetLastError());
		return 1;
	}	

	// configure the J1708 port
	// see section 7.2 of hardware manual for details of parameters
	j1708Config.mask_good_tx	= 0;
	j1708Config.Tx_Priority		= 7;
	j1708Config.Tx_Problem_Limit = 10;
	j1708Config.Rx_Afull_Level	= 16;
	j1708Config.Rx_EOM_Level	= 6;
	if( SetupJ1708( threadContext.hDriver, &j1708Config) )
	{
		printf("SetupJ1708 failed");
		goto err_exit;
	}

	if( receiveOnly == TRUE || (receiveOnly == FALSE && transmitOnly == FALSE))
	{
		hReadThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) ReadThreadJ1708,
			(LPVOID) &threadContext, 0, &dwThreadStatId);
		if (!hReadThread)                 
		{		
			printf("CreateThread ReadThreadJ1708 failed, error %x\n", GetLastError());
			goto err_exit;
		}
	}

	if( transmitOnly == TRUE || (receiveOnly == FALSE && transmitOnly == FALSE ))
	{
		hWriteThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) WriteThreadJ1708,
			(LPVOID) &threadContext, 0, &dwThreadStatId);
		if (!hWriteThread)                 
		{		
			printf("CreateThread WriteThreadJ1708 failed, error %x\n", GetLastError());
			goto err_exit;
		}
	}

	while(1)
	{						
		Sleep(1000*5);		
	}

err_exit:
	if(threadContext.hDriver)
		CloseHandle(threadContext.hDriver);

	return 0;
}

/***************************************************************************
* WriteThreadJ1708 
*
* 
* thread for performing transmit operations and checking errors
*
* Arguments:
*	pContext	: pointer to unit_CONTEXT
*	
*
* Returns:
*   1
*   
*/
DWORD WINAPI
 WriteThreadJ1708(
    PUNIT_CONTEXT	pContext
    )
{
	char	BUF[MAX_BUF_SIZE];
	char	*buffer;
	ULONG	ret;
	int		i, bufSize;
	ULONG	writeStatus;
	#ifdef USE_FILE
	FILE *fp;
	#endif
	

	#ifdef USE_FILE
	fp = fopen("test.txt","rt");
	if(!fp)
	{
		printf("fopen failed\n");
		return 0;
	}
	#else
	// create A,B,C etc repeated in test buffer
	for (i = 0; i < (int)MAX_BUF_SIZE; i++)
        BUF[i] = (unsigned char) (i % 26 + 0x41);
	#endif

	while(1)
	{			
		bufSize = BUF_SIZE;
		buffer = BUF;

		if(!bufSize)
		{				
			printf("J1708 buffer size of 0 is invalid\n");
			break;
		}

start_tx:
		#ifdef USE_FILE
		bufSize = fread(buffer, 1, BUF_SIZE, fp);
		if(feof(fp))
			exit(1);				
		#endif
		ret = WriteJ1708(pContext->hDriver, buffer, bufSize);
		if( ret == J1708_STATUS_OK )
		{
			// must call GetStatus after successful Write to verify packet sent without errors
			ret = GetWriteStatusJ1708( pContext->hDriver, (char *)&writeStatus, sizeof writeStatus );
			if( ret == J1708_STATUS_OK )
			{
				if ((writeStatus & STATUS_J1708_TX_ABORTED) == STATUS_J1708_TX_ABORTED)
				{
					printf("J1708 current packet transmission aborted\n");
				}
				if((writeStatus & STATUS_J1708_TX_PROB) == STATUS_J1708_TX_PROB)
				{
					printf("J1708 Tx problem limit, abort current packet transmission\n");
					// abort the current packet transmission
					if( !AbortJ1708TX( pContext->hDriver ) )
					{
						printf("J1708 AbortJ1708TX failed");						
					}
				}
			}
			else
			{
				printf("J1708 GetStatusJ1708 failed error code = %d\n", ret);
				return 1;
			}

			
			if(pContext->delay)
				Sleep(pContext->delay);
			else // give time slice to other thread
				SwitchToThread();				
			goto start_tx;
		}
		else
		{
			printf("J1708 WriteJ1708 failed error code = %d\n", ret);
			return 1;
		}

		
	}
	return 1;
}


/***************************************************************************
* ReadThreadJ1708 
*
* 
* thread that reads a message from a unit and handles errors.
*
* Arguments:
*	pContext	: pointer to unit_CONTEXT
*	
*
* Returns:
*   1
*   
*/
DWORD WINAPI
 ReadThreadJ1708(
    PUNIT_CONTEXT	pContext
    )
{		
	char	buffer[MAX_BUF_SIZE];
	ULONG	ret, bufReq;
	int		numRead, i;
	ULONG	readStatus;
	
	while(1)	
	{		
		bufReq = BUF_SIZE;
ReadTryAgain:
		ret = ReadJ1708(pContext->hDriver, buffer, bufReq, &numRead);
		if (ret == J1708_STATUS_OK)
		{
			// check for read errors, GetReadStatus resets error			
			ret = GetReadStatusJ1708( pContext->hDriver, (char *)&readStatus, sizeof readStatus );			
			if( ret == J1708_STATUS_OK )
			{
				if( (readStatus & BUF_OVERRUN) == BUF_OVERRUN )
				{
					printf("J1708 BUF_OVERRUN error\n");					
				}
			}
			else
			{
				printf("J1708 GetReadStatusJ1708 error %d\n", ret);
				return 1;
			}
		}
		else
		{
			if (ret != J1708_STATUS_INSUFFICIENT_BUFFER)
			{
				printf("J1708 ReadJ1708 error %d\n", ret);	
				return 1;
			}

		}
		if(numRead)
		{			
			// display data that has been received
			for(i = 0; i < numRead; i++)
				printf("%c",buffer[i]);			
		}
		if (ret == J1708_STATUS_INSUFFICIENT_BUFFER || readStatus & BUF_OVERRUN)
		{
			int lastStatus = ret;
			bufReq = 0;			
			// buffer was not big enough so find out how much space is needed and try again
			ret = GetReadDataAvailJ1708(pContext->hDriver, &bufReq);
			if( ret == J1708_STATUS_OK )
			{
				// ensure we have enough buffer available to retry with larger size
				if( bufReq > MAX_BUF_SIZE )					
				{
					printf("J1708 GetReadDataAvailJ1708 error, buffer required bigger than maximum size\n");
					bufReq = MAX_BUF_SIZE;
				}
				
				if(lastStatus == J1708_STATUS_INSUFFICIENT_BUFFER)
				{					
					goto ReadTryAgain;			
				}
			}
			else
			{
				printf("J1708 GetReadDataAvailJ1708 error %d\n", ret);
				return 1;
			}
		}
		// if there is nothing to be done yield execution to other threads
		if(!numRead)	
		{
			SwitchToThread();
			continue;
		}
		
		if(pContext->delay)
		{
			// see whether more packets are available, read them otherwise overrun will happen
			ret = GetReadDataAvailJ1708(pContext->hDriver, &bufReq);
			if( ret == J1708_STATUS_OK )
			{
				// ensure we have enough buffer available to retry with larger size
				if( bufReq > MAX_BUF_SIZE )					
				{
					printf("J1708 GetReadDataAvailJ1708 error, buffer required bigger than maximum size\n");
					bufReq = MAX_BUF_SIZE;
				}
				
				if(bufReq)			
					goto ReadTryAgain;							
			}
			else
				return 1;
			Sleep(pContext->delay);
		}
		else // give time slice to other thread
			SwitchToThread();	
	}

	return 1;
}

